#include "CC1020.h"

/**
 * This routine calibrates the CC1020.  It should be called for both receive and transmit states.
 * 
 * @pre All configuration registers have been populated and the CC1020 has been either placed in
 *      transmit or receive mode.
 * 
 * @post The CC1020 is calibrated for either transmit or receive mode.
 * 
 * @return 0 if calibration fails, non-zero otherwise. Checks the LOCK to check for success
 */
char CalibrateCC1020(void)
{
  volatile int TimeOutCounter;
  volatile int nCalAttempt;

  // Turn off PA to avoid spurs during calibration in TX mode
  WriteToCC1020Register(CC1020_PA_POWER,0x00);

  // Calibrate, and re-calibrate if necessary:
  for (nCalAttempt = 2 /*CAL_ATTEMPT_MAX*/; (nCalAttempt>0); nCalAttempt--) {

    // Start calibration
    WriteToCC1020Register(CC1020_CALIBRATE,0xB4);

    // Monitor actual calibration start (ref. Errata Note 04 - CC1020)
    for(TimeOutCounter=CAL_TIMEOUT; ((ReadFromCC1020Register(CC1020_STATUS)&0x80)==0x80)&&(TimeOutCounter>0); TimeOutCounter--);
    /**
     * @remarks 
     * Important note from Chipcon:
     * In active mode the CC1020 should theoretically initiate an internal action/process more or less
     * instantly upon receiving any command from e.g. an MCU. However, upon sending a [calibration start]
     * command to CC1020, tests shows that the [STATUS.CAL_COMPLETE]-signal sometimes remains asserted
     * (or idle) close to 100 usec after the command has been originally issued. Consequently this process
     * must be carefully monitored to avoid premature PLL LOCK monitoring; do not proceed with subsequent
     * PLL LOCK monitoring before the calibration has actually completed inside the CC1020! Errata Note 04
     * suggests that [calibration start] should be monitored by a fixed timeout > 100 usec. However, the
     * above method implements an adaptive monitoring of [calibration start], which basically waits for the
     * [STATUS.CAL_COMPLETE]-signal to initialise/deassert (indicating calibration has actually started)
     * before proceeding with monitoring calibration complete and PLL LOCK. Chipcon considers both methods
     * safe, and thus leaves it up to the user, which one to use.
     */

    // Monitor calibration complete
    for(TimeOutCounter=CAL_TIMEOUT; ((ReadFromCC1020Register(CC1020_STATUS)&0x80)==0x00)&&(TimeOutCounter>0); TimeOutCounter--);

    // Monitor lock
    for(TimeOutCounter=LOCK_TIMEOUT; ((ReadFromCC1020Register(CC1020_STATUS)&0x10)==0x00)&&(TimeOutCounter>0); TimeOutCounter--);

    // Abort further recalibration attempts if successful LOCK
    if((ReadFromCC1020Register(CC1020_STATUS)&0x10) == 0x10) {
      break;
    }
  }

  // Restore PA setting
  WriteToCC1020Register(CC1020_PA_POWER, PA_POWER);

  // Return state of LOCK_CONTINUOUS bit
  return ((ReadFromCC1020Register(CC1020_STATUS)&0x10)==0x10);
}

